/*
 * coordinate_transform.hpp
 *
 *  Created on: Sep 4, 2017
 *      Author: julien
 */

#ifndef SRC_MODULES_COORDINATE_TRANSFORM_COORDINATE_TRANSFORM_HPP_
#define SRC_MODULES_COORDINATE_TRANSFORM_COORDINATE_TRANSFORM_HPP_

#include "../../OpenCV_includes.hpp"
#include "../Function/Function.hpp"


class CoordinateTransform{

	private:

		float camera_height;
		float height_squared;
		float camera_angle;
		float aperture_angle_x;
		float aperture_angle_y;
		float picture_height;
		float picture_width;

		Function::pPoint camera_translation;

		Function f;




	public:
		CoordinateTransform(float camera_height,
							float camera_angle,
							float aperture_angle_x,
							float aperture_angle_y,
							Size picture_size,
							Function::pPoint camera_translation);

		/**
		 * TODO
		 */
		void transform(vector<Function::pPoint>* input_l,
					   vector<Function::pPoint>* ouput_l,
					   vector<Function::pPoint>* input_r,
					   vector<Function::pPoint>* ouput_r,
					   bool input_is_pixel);

		/**
		 * transforms a single Point from picture coordinates into real world coordinates
		 */
		Point pixel2real(Point input);

		/**
		 * transforms real world coordinates into pixel coordinates
		 */
		Point real2pixel(Point input);


		/**
		 * transforms a point in standard pixel coordinates (columns and rows) into central ego coordinates
		 * From:			Into:
		 * +--------->		       ^x
		 * |		x		       | (this is in SI-Units)
		 * |				y	   |
		 * |				<------+-------
		 * |y					   |
		 * v					   |
		 */
		Function::pPoint pixel2ego_coord(Point pixel_point);

		Function::pPoint ego_coord2pixel(Function::pPoint ego_point);


		/**
		 * transforms a point in standard pixel coordinates (columns and rows) into bottom Pixel coordinates
		 * This is used for taking measurements from the picture
		 * From:			Into:
		 * +--------->		       ^y
		 * |		x		       |
		 * |					   |
		 * |					   |
		 * |y					   |	  x
		 * v				-------+------>
		 */
		Point pixel2bottom_coord(Point pixel_point);

		Point bottom_coord2pixel(Point bottom_point);

		void transf(vector<Function::pPoint>* input_lane,
			        vector<Function::pPoint>* ouput_lane,
				    bool input_is_pixel);

};



#endif /* SRC_MODULES_COORDINATE_TRANSFORM_COORDINATE_TRANSFORM_HPP_ */
